/**************************************************************************//**
*   @file   dma.c
*   @brief  Implementation of the DMA transfer functions.
*   @author ACozma (andrei.cozma@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: $WCREV$
******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <xil_io.h>
#include <xparameters.h>
#include "dma.h"

/**************************************************************************//**
* @brief Transfers data using SG DMA from memory to a streaming peripheral.
*
* @param pCurrentDescriptor - Current descriptor pointer.
* @param pTailDescriptor - Tail descriptor pointer.
* @return None
******************************************************************************/
void DMA_TransferDataTX(stTransferDescriptor* pCurrentDescriptor, 
						stTransferDescriptor* pTailDescriptor)
{
	//stop DMA
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x00), 0x00);
	//set the current descriptor pointer
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x08), (u32)pCurrentDescriptor);
    //start DMA
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x00), 0x01);
	//set the tail descriptor pointer
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x10), (u32)pTailDescriptor);
}

/**************************************************************************//**
* @brief Checks if the DMA transfer from memory to a streaming peripheral 
*        is complete.
*
* @return Returns 0 is the transfer is in progress, 1 if the transfer is cmmplete.
******************************************************************************/
u32 DMA_IsTransferCompleteTX(void)
{
	return Xil_In32((XPAR_AXIDMA_0_BASEADDR + 0x04)) & 0x02;
}

/**************************************************************************//**
* @brief Transfers data using SG DMA from a streaming peripheral to memory.
*
* @param pCurrentDescriptor - Current descriptor pointer.
* @param pTailDescriptor - Tail descriptor pointer.
* @return None
******************************************************************************/
void DMA_TransferDataRX(stTransferDescriptor* pCurrentDescriptor, 
						stTransferDescriptor* pTailDescriptor)
{
	//stop DMA
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x30), 0x00);
	//set the current descriptor pointer
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x38), (u32)pCurrentDescriptor);
    //start DMA
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x30), 0x01);
	//set the tail descriptor pointer
	Xil_Out32((XPAR_AXIDMA_0_BASEADDR + 0x40), (u32)pTailDescriptor);

}

/**************************************************************************//**
* @brief Checks if the DMA transfer from a streaming peripheral to memory
*        is complete.
*
* @return Returns 0 is the transfer is in progress, 1 if the transfer is cmmplete.
******************************************************************************/
u32 DMA_IsTransferCompleteRX(void)
{
	return Xil_In32((XPAR_AXIDMA_0_BASEADDR + 0x34)) & 0x02;
}
